﻿Imports System.ComponentModel
Imports System.Text

Imports Ozeki.Media
Imports Ozeki.Media.MediaHandlers
Imports Ozeki.Network.Nat
Imports Ozeki.VoIP
Imports Ozeki.VoIP.Media
Imports Ozeki.VoIP.SDK
Imports Ozeki.Media.MediaHandlers.Facade
Imports Ozeki.Common
Imports System.Configuration
Imports System.IO

Namespace DTMFIVRTreeNav
'''    
'''     * This class demonstrates how you can navigate in an IVR tree using DTMF signals. It contains a softphone engine, that
'''     * registers to the PBX with a SIP account specified in the app.config file and automaically answers the incoming calls.
'''     * The program uses text to speech conversion for reading out the actual IVR node information.
'''     * When a call is answered the IVR tree root information will be read out. If a DTMF signal arrives, the system decides on
'''     * stepping into a new IVR tree branch or playing the actual node information again.
'''     * 
	Partial Public Class Softphone
		Inherits Form
'INSTANT VB NOTE: The variable softPhone was renamed since Visual Basic does not allow class members with the same name:
		Private softPhone_Renamed As ISoftPhone
		Private phoneLine As IPhoneLine
		Private phoneLineInformation As PhoneLineState
		Private [call] As IPhoneCall
		'Microphone microphone = Microphone.GetDefaultDevice();
		Private speaker As Speaker = Speaker.GetDefaultDevice()
		Private connector As New MediaConnector()
		Private mediaSender As New PhoneCallAudioSender()
		Private mediaReceiver As New PhoneCallAudioReceiver()
		Private ivrReader As New TextToSpeech()

		'SIP account tools
		Private IsRegRequired As Boolean
		Private displayname As String
		Private username As String
		Private registername As String
		Private regpass As String
		Private domainhost As String
		Private port As Integer
		Private natTraversal As NatTraversalMethod
		Private root As IVRNode
		Private selectedNode As IVRNode
		Private selectedTreeNode As TreeNode
		Private atRoot As Boolean = True

		Public Sub New()
			InitializeComponent()
			buildTree()

		End Sub

		Private Sub buildTree()
			root = TreeXMLParser.parse(TreeXMLParser.treedoc, Nothing)
			treeView1.Nodes.Add(root.treeNode)
			selectedNode = root
			selectedTreeNode = root.treeNode
			Try
				treeView1.SelectedNode = selectedTreeNode
				treeView1.SelectedNode.ForeColor = Color.Red
			Catch e1 As Exception

				MessageBox.Show("Error", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
			End Try
		End Sub

		#Region "Ozeki VoIP-SIP SDK's events"

		''' <summary>
		''' Occurs when phone line state has changed.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub phoneLine_PhoneLineInformation(ByVal sender As Object, ByVal e As VoIPEventArgs(Of PhoneLineState))
			phoneLineInformation = e.Item
			InvokeGUIThread(Sub()
				label2.Text = CType(sender, IPhoneLine).SIPAccount.RegisterName
				If e.Item = PhoneLineState.RegistrationSucceeded Then
					label3.Text = "Registration succeeded"
					label1.Text = "Online"
				Else
					Informationallabel.Text = e.Item.ToString()
				End If
			End Sub)
		End Sub

		''' <summary>
		''' Occurs when an incoming call request has received.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub softPhone_IncomingCall(ByVal sender As Object, ByVal e As VoIPEventArgs(Of IPhoneCall))
			InvokeGUIThread(Sub()
				'inComingCall = true;
				[call] = e.Item
				WireUpCallEvents()
				[call].Accept()
			End Sub)
		End Sub

		''' <summary>
		''' Occurs when the phone call state has changed.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		''' 

		Private Sub call_CallStateChanged(ByVal sender As Object, ByVal e As VoIPEventArgs(Of CallState))
			InvokeGUIThread(Sub() label1.Text = e.Item.ToString())

			Select Case e.Item
				Case CallState.InCall

					connector.Connect(ivrReader, mediaSender)

					mediaSender.AttachToCall([call])
					mediaReceiver.AttachToCall([call])

					ivrReader.AddAndStartText(selectedNode.message)

				Case CallState.Completed

					connector.Disconnect(ivrReader, mediaSender)


					mediaSender.Detach()
					mediaReceiver.Detach()

					WireDownCallEvents()
					[call] = Nothing

				Case CallState.Cancelled

					WireDownCallEvents()
					[call] = Nothing
			End Select
		End Sub

		''' <summary>
		''' Displays DTMF signals
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub call_DtmfReceived(ByVal sender As Object, ByVal e As VoIPEventArgs(Of OzTuple(Of VoIPMediaType, DtmfSignal)))
			InvokeGUIThread(Sub() label1.Text = String.Format("DTMF signal received: {0} ", e.Item.Item2.Signal))
			'you can restart the actual message by pressing star
			If e.Item.Item2.Signal = 10 Then
				ivrReader.StopStreaming()
				ivrReader.AddAndStartText(selectedNode.message)
			End If
			'pressing hash gets you back to the main menu
			If e.Item.Item2.Signal = 11 AndAlso (Not atRoot) Then
				atRoot = True
				selectedNode = root
				selectedTreeNode = root.treeNode
				InvokeGUIThread(Sub()
					treeView1.SelectedNode.ForeColor = Color.Black
					treeView1.SelectedNode = selectedTreeNode
					treeView1.SelectedNode.ForeColor = Color.Red
				End Sub)
				ivrReader.StopStreaming()
				ivrReader.AddAndStartText(selectedNode.message)
			End If
			'stepping into a submenu if exists
			If selectedNode.getChild(e.Item.Item2.Signal) IsNot Nothing Then
				atRoot = False
				ivrReader.StopStreaming()
				selectedNode = selectedNode.getChild(e.Item.Item2.Signal)
				selectedTreeNode = selectedNode.treeNode
				InvokeGUIThread(Sub()
					treeView1.SelectedNode.ForeColor = Color.Black
					treeView1.SelectedNode = selectedTreeNode
					treeView1.SelectedNode.ForeColor = Color.Red
				End Sub)

				ivrReader.AddAndStartText(selectedNode.message)
			End If
		End Sub

		''' <summary>
		''' There are certain situations when the call cannot be created, for example the dialed number is not available 
		''' or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
		''' This event handling is for displaying these events.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub call_CallErrorOccured(ByVal sender As Object, ByVal e As VoIPEventArgs(Of CallError))
			InvokeGUIThread(Sub() label1.Text = e.Item.ToString())
		End Sub

		#End Region


		#Region "Helper Functions"
		''' <summary>
		'''It initializes a softphone object with a SIP BPX, and it is for requisiting a SIP account that is nedded for a SIP PBX service. It registers this SIP
		'''account to the SIP PBX through an ’IphoneLine’ object which represents the telephoneline. 
		'''If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
		''' </summary>
		Private Sub InitializeSoftPhone()

				softPhone_Renamed = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700)
				AddHandler softPhone_Renamed.IncomingCall, AddressOf softPhone_IncomingCall
				register()
		End Sub

		Private Sub register()
			Try
				Dim cm = ConfigurationManager.AppSettings
				IsRegRequired = Boolean.Parse(cm("IsRegRequired"))
				displayname = cm("displayname")
				username = cm("username")
				registername = cm("registername")
				regpass = cm("regpass")
				domainhost = cm("domainhost")
				port = Int32.Parse(cm("port"))

				Select Case Int32.Parse(cm("nat"))
					Case 0
						natTraversal = NatTraversalMethod.None
					Case 1
						natTraversal = NatTraversalMethod.STUN
					Case 2
						natTraversal = NatTraversalMethod.TURN
				End Select

				Dim sa As New SIPAccount(IsRegRequired, displayname, username, registername, regpass, domainhost, port)
				Dim nc As New NatConfiguration(natTraversal)

				phoneLine = softPhone_Renamed.CreatePhoneLine(sa, nc)
				AddHandler phoneLine.PhoneLineStateChanged, AddressOf phoneLine_PhoneLineInformation

				softPhone_Renamed.RegisterPhoneLine(phoneLine)
			Catch ex As Exception
				MessageBox.Show(String.Format("You didn't set the right IP address. Please set your IP address and try again" & vbLf & " {0}", ex.Message), String.Empty, MessageBoxButtons.OK, MessageBoxIcon.Error)
			End Try
		End Sub

		''' <summary>
		'''  It signs up to the necessary events of a call transact.
		''' </summary>
		Private Sub WireUpCallEvents()
			AddHandler [call].CallStateChanged, AddressOf call_CallStateChanged
			AddHandler [call].DtmfReceived, AddressOf call_DtmfReceived
			AddHandler [call].CallErrorOccured, AddressOf call_CallErrorOccured
		End Sub

		''' <summary>
		''' It signs down from the necessary events of a call transact.
		''' </summary>
		Private Sub WireDownCallEvents()
			If [call] IsNot Nothing Then
                RemoveHandler [call].CallStateChanged, (AddressOf call_CallStateChanged)
                RemoveHandler [call].DtmfReceived, (AddressOf call_DtmfReceived)
                RemoveHandler [call].CallErrorOccured, (AddressOf call_CallErrorOccured)
			End If

		End Sub

		''' <summary>
		''' The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
		''' </summary>
		''' <param name="action"></param>
		Private Sub InvokeGUIThread(ByVal action As Action)
			Invoke(action)
		End Sub
		#End Region

        Private Sub Softphone_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
            InitializeSoftPhone()

        End Sub
    End Class
End Namespace
